# -----------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -----------------------------------------------------------------------------
"""
Download command.
"""

from paconn.common.util import display
from paconn.common.settings_util import load_settings
from paconn import _DOWNLOAD

from .save import save


# pylint: disable=too-many-arguments
def download(environment, connector_id, destination, powerapps_url, powerapps_version, settings_file):
    """
    Download command.
    """
    settings, powerapps_rp = load_settings(
        environment=environment,
        settings_file=settings_file,
        connector_id=connector_id,
        powerapps_url=powerapps_url,
        powerapps_version=powerapps_version,
        command_context=_DOWNLOAD)

    directory = save(
        powerapps_rp=powerapps_rp,
        settings=settings,
        destination=destination)

    display('The connector is downloaded to {}.'.format(directory))



# -----------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -----------------------------------------------------------------------------
"""
Represents a settings object consructed from settings.json
"""


# pylint: disable=too-few-public-methods
class Settings:
    """
    Represents a settings object consructed from settings.json
    """
    # pylint: disable=too-many-arguments
    def __init__(
            self,
            connector_id,
            environment,
            powerapps_url,
            powerapps_api_version,
            api_properties,
            api_definition,
            icon):
        self.connector_id = connector_id
        self.environment = environment
        self.powerapps_url = powerapps_url
        self.powerapps_api_version = powerapps_api_version
        self.api_properties = api_properties
        self.api_definition = api_definition
        self.icon = icon



# -----------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -----------------------------------------------------------------------------
"""
Utility for loading settings.
"""

from paconn import _CREATE, _DOWNLOAD
from paconn.apimanager.apimanager import APIManagerBuilder
from paconn.apimanager.powerappsrp import PowerAppsRP
from paconn.authentication.tokenmanager import TokenManager

from .settings import Settings
from .settings_serializer import SettingsSerializer
from .prompts import get_environment, get_connector_id

# PowerApps URL
_DEFAULT_POWERAPPS_URL = 'https://preview.api.powerapps.com'
_DEFAULT_POWERAPPS_BASE_PATH = 'providers/Microsoft.PowerApps'
_DEFAULT_POWERAPPS_API_VERSION = '2016-11-01'

# File names
API_PROPERTIES_FILE = 'apiProperties.json'
API_DEFINITION_FILE = 'apiDefinition.swagger.json'
ICON_FILE = 'icon.png'
SETTINGS_FILE = 'settings.json'


# pylint: disable=too-many-arguments
def load_settings(
        environment,
        settings_file,
        connector_id,
        powerapps_url,
        powerapps_version,
        command_context,
        api_properties=API_PROPERTIES_FILE,
        api_definition=API_DEFINITION_FILE,
        icon=ICON_FILE):
    """
    Loads settings into a settings object.
    """
    # Get Authentication Token
    tokenmanager = TokenManager()
    credentials = tokenmanager.get_credentials()

    if settings_file:
        settings = SettingsSerializer.from_json(settings_file)
    else:
        settings = Settings(
            connector_id=connector_id,
            environment=environment,
            powerapps_url=powerapps_url,
            powerapps_api_version=powerapps_version,
            api_properties=api_properties,
            api_definition=api_definition,
            icon=icon
        )

    if not hasattr(settings, 'powerapps_url') or not settings.powerapps_url:
        settings.powerapps_url = _DEFAULT_POWERAPPS_URL
    if not hasattr(settings, 'powerapps_api_version') or not settings.powerapps_api_version:
        settings.powerapps_api_version = _DEFAULT_POWERAPPS_API_VERSION

    if command_context is _DOWNLOAD:
        if not hasattr(settings, 'api_properties') or not settings.api_properties:
            settings.api_properties = api_properties
        if not hasattr(settings, 'api_definition') or not settings.api_definition:
            settings.api_definition = api_definition
        if not hasattr(settings, 'icon') or not settings.icon:
            settings.icon = icon

    # Select environment if not provided
    if not hasattr(settings, 'environment') or not settings.environment:
        settings.environment = get_environment(
            credentials=credentials)

    # Create the API Manager
    powerapps_api_manager = APIManagerBuilder.get_from_url(
        url=settings.powerapps_url,
        base_path=_DEFAULT_POWERAPPS_BASE_PATH,
        api_version=settings.powerapps_api_version,
        credentials=credentials)

    powerapps_rp = PowerAppsRP(api_manager=powerapps_api_manager)

    # Select connector id if not provided
    if (not hasattr(settings, 'connector_id') or not settings.connector_id) and command_context is not _CREATE:
        settings.connector_id = get_connector_id(
            powerapps_rp=powerapps_rp,
            environment=settings.environment)

    return settings, powerapps_rp


def save_settings(settings, settings_file=SETTINGS_FILE):
    """
    Saves settings.
    """
    # Write the setings.json
    if settings:
        SettingsSerializer.to_json(settings, settings_file)



# -----------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -----------------------------------------------------------------------------
"""
Represents a settings object consructed from settings.json
"""

import json
from paconn.common.util import format_json
from .settings import Settings

_CONNECTOR_ID = 'connectorId'
_ENVIRONMENT = 'environment'
_POWERAPPS_URL = 'powerAppsUrl'
_POWERAPPS_API_VERSION = 'powerAppsApiVersion'
_API_PROPERTIES = 'apiProperties'
_API_DEFINITION = 'apiDefinition'
_ICON = 'icon'


# pylint: disable=too-few-public-methods
class SettingsSerializer:
    """
    Serializes and deserializes a settings object
    """
    @staticmethod
    def to_json(settings, filename):
        """
        Serializes a settings object into the settings.json
        """
        settings_dict = SettingsSerializer.serialize(settings)
        json_str = format_json(settings_dict)
        open(filename, 'w').write(json_str)

    @staticmethod
    def from_json(filename):
        """
        Deserializes a settings object from the settings.json file
        """
        with open(filename, 'r') as file:
            settings_dict = json.load(file)  # pylint: disable=attribute-defined-outside-init
        settings = SettingsSerializer.deserialize(settings_dict)
        return settings

    @staticmethod
    def serialize(settings):
        """
        Serializes a settings object into a dict
        """
        settings_dict = {
            _CONNECTOR_ID: settings.connector_id,
            _ENVIRONMENT: settings.environment,
            _POWERAPPS_URL: settings.powerapps_url,
            _POWERAPPS_API_VERSION: settings.powerapps_api_version,
            _API_PROPERTIES: settings.api_properties,
            _API_DEFINITION: settings.api_definition,
            _ICON: settings.icon
        }
        return settings_dict

    @staticmethod
    def deserialize(settings_dict):
        """
        Deserializes a dictionary to a settings object
        """
        settings = Settings(
            connector_id=settings_dict.get(_CONNECTOR_ID, None),
            environment=settings_dict.get(_ENVIRONMENT, None),
            powerapps_url=settings_dict.get(_POWERAPPS_URL, None),
            powerapps_api_version=settings_dict.get(_POWERAPPS_API_VERSION, None),
            api_properties=settings_dict.get(_API_PROPERTIES, None),
            api_definition=settings_dict.get(_API_DEFINITION, None),
            icon=settings_dict.get(_ICON, None)
        )
        return settings
